library(flexsurv)

S = matrix(c(0.1,0.6,1),3,1) #setting of standardized stress levels
G = matrix(c(0.61,1.55,10.22,-0.8)) #setting of GG_3 distribution

# compare the MTTF
K.g = (G[3,1])
sigma.g = 1/G[1,1]
r0g = log(G[2,1]) 
m.g = (r0g + (S)*G[4,1]) 
(m.g + sigma.g*digamma((K.g)))

###################################################

# SEV teta star; Teoritical result
neg.loglike.e  = function(teta) {
  
  log.beta= teta[3]; sigma = exp(log.beta)
  log.r0  = teta[1]; r0    = exp(log.r0)
  log.r1  = teta[2]; r1    = (log.r1)
  
  mu = r0 + r1*S
  g = exp(lgamma(sigma.g/sigma + K.g))/exp(lgamma(K.g))
  
  (4/7)*log(sigma) + (4/7)*(mu[1,] - m.g[1,] - sigma.g*digamma(K.g))/sigma + (4/7)*g*sum(exp(-(mu[1,] - m.g[1,])/sigma))+
    (2/7)*log(sigma) + (2/7)*(mu[2,] - m.g[2,] - sigma.g*digamma(K.g))/sigma + (2/7)*g*sum(exp(-(mu[2,] - m.g[2,])/sigma))+
    (1/7)*log(sigma) + (1/7)*(mu[3,] - m.g[3,] - sigma.g*digamma(K.g))/sigma + (1/7)*g*sum(exp(-(mu[3,] - m.g[3,])/sigma))
}
fitt.e   = optim(c(1,0,1),neg.loglike.e, hessian = TRUE)

(sigmae = exp(fitt.e$par[3]))
(a0.e  = exp(fitt.e$par[1]))
(a1.e  = (fitt.e$par[2]))

#Normal theta.star
neg.loglike.n  = function(teta) {
  log.beta= teta[3]; sigma = exp(log.beta)
  log.r0  = teta[1]; r0    = exp(log.r0)
  log.r1  = teta[2]; r1    = (log.r1)
  
  mu = r0 + r1*S
  
  (4/7)*log(sigma) + (4/7)*((mu[1,] - m.g[1,] - sigma.g*digamma(K.g))^2 + sigma.g^2*trigamma(K.g))/(2*(sigma)^2)+
    (2/7)*log(sigma) + (2/7)*((mu[2,] - m.g[2,] - sigma.g*digamma(K.g))^2 + sigma.g^2*trigamma(K.g))/(2*(sigma)^2)+
    (1/7)*log(sigma) + (1/7)*((mu[3,] - m.g[3,] - sigma.g*digamma(K.g))^2 + sigma.g^2*trigamma(K.g))/(2*(sigma)^2)
  
}
fitt.n  = optim(c(1,0,1), neg.loglike.n, hessian = TRUE)
(sigman = exp(fitt.n$par[3]))
(a0.n  = exp(fitt.n$par[1]))
(a1.n  = (fitt.n$par[2]))


# Plot 3 & 4
c = seq(0.01,0.99, by =0.01)
v1 = qgengamma.orig(c, G[1,1],G[2,1] ,G[3,1])
v2 = qlnorm(c, mean = a0.n , sd = sigman)
v3 = qweibull(c, 1/sigmae, exp(a0.e))

Rbiasgeneral = matrix(c(rep(0,99), (v3-v1)/v1,(v2-v1)/v1 ),99,3)
Rbiasgenerals = matrix(c(rep(0,99), (v3-v1)^2,(v2-v1)^2 ),99,3)

par(mfrow =c(1,2))

matplot(cbind(Rbiasgeneral[,1],Rbiasgeneral[,2]), type ="l", pch = 20, xlab = "(a)   100% p", ylab = "RB", cex.lab=1.1, cex.axis=1.1, main = "Weibull", ylim = c(-0.5, 0.2))
matplot(cbind(Rbiasgeneral[,1],Rbiasgeneral[,3]), type ="l", pch = 20, xlab = "(b)   100% p", ylab = "RB", cex.lab=1.1, cex.axis=1.1, main = "Lognormal", ylim = c(-0.5, 0.2))

matplot(cbind(Rbiasgenerals[,2]), type ="l", pch = 20, xlab = "(a)   100% p", ylab = "RB", cex.lab=1.1, cex.axis=1.1, main = "Weibull")
matplot(cbind(Rbiasgenerals[,3]), type ="l", pch = 20, xlab = "(b)   100% p", ylab = "RB", cex.lab=1.1, cex.axis=1.1, main = "Lognormal")




###################################
#simulation

q = 1000 # of loop, repeated trail
aa = 5
r0.g  = matrix(0,q,aa)
r1.g  = matrix(0,q,aa)
sig.g = matrix(0,q,aa)
k     = matrix(0,q,aa)
sig.n = matrix(0,q,aa)
r0.n  = matrix(0,q,aa)
r1.n  = matrix(0,q,aa)
r0.e  = matrix(0,q,aa)
r1.e  = matrix(0,q,aa)
sig.e = matrix(0,q,aa)
x = matrix(c(5,10,20,40,63))
a = 0
for ( p in 1:aa){
  for ( h in x[p,]){
    if ( a > aa) {break}
    sn = matrix(c(4*h,2*h,1*h),3,1)
    m1 = matrix(0,q,sn[1,])
    m2 = matrix(0,q,sn[2,])
    m3 = matrix(0,q,sn[3,])
    a = a + 1
    i = 0
    repeat {
      i <- i + 1
      if ( i > q) {break}
      m1[i,] = rgengamma.orig(sn[1,], G[1,1] , scale = G[2,1]*exp(S[1,]*G[4,1]), G[3,1])
      m2[i,] = rgengamma.orig(sn[2,], G[1,1] , scale = G[2,1]*exp(S[2,]*G[4,1]), G[3,1])
      m3[i,] = rgengamma.orig(sn[3,], G[1,1] , scale = G[2,1]*exp(S[3,]*G[4,1]), G[3,1])
      
      data = c(m1[i,], m2[i,], m3[i,])
      cen = rep(1,sn[1,]+sn[2,]+sn[3,])
      level = c(rep(S[1,],sn[1,]), rep(S[2,],sn[2,]), rep(1,sn[3,]))
      
      gam <- flexsurvreg(formula = Surv((data), cen) ~ (level), dist="gengamma.orig", inits = c(G[1,1],G[2,1],G[3,1],G[4,1]), fixedpars = c(1,3))$res
      (sig.g[i,a]  = gam[1,1])
      (r0.g[i,a]   = gam[2,1])
      (r1.g[i,a]   = gam[4,1])
      (k[i,a]      = gam[3,1])
      
      nor<- flexsurvreg(formula = Surv((data), cen) ~ level, dist="lnorm", inits = c(a0.n,sigman,a1.n), fixedpars = c(2))$res
      (sig.n[i,a] = nor[2,1])
      (r0.n[i,a] = nor[1,1])
      (r1.n[i,a] = nor[3,1])
      
      wei<- flexsurvreg(formula = Surv((data), cen) ~ level, dist="weibull", inits = c(1/sigmae,exp(a0.e),a1.e), fixedpars = c(1))$res
      (sig.e[i,a] = wei[1,1])
      (r0.e[i,a] = wei[2,1])
      (r1.e[i,a] = wei[3,1])
      
      if (i%%100==0) print(paste("This is iteration number ", i*a))
    }
  }
}

#compute quantile
j = 99 
msg    = array(0,dim=c(q,j,aa))
msn    = array(0,dim=c(q,j,aa))
msw    = array(0,dim=c(q,j,aa))
RVg    = matrix(0,j,aa)
RVn    = matrix(0,j,aa)
RVw    = matrix(0,j,aa)
RBN   = matrix(0,j,aa)
RBW   = matrix(0,j,aa)
RVwg  = matrix(0,j,aa)
RVng  = matrix(0,j,aa)
t.g    = array(0,dim=c(q,j,aa))
t.w    = array(0,dim=c(q,j,aa))
t.n    = array(0,dim=c(q,j,aa))
t.ghat = matrix(0,j,aa)
t.nbar = matrix(0,j,aa)
t.wbar = matrix(0,j,aa)
t.nsqs = array(0,dim=c(j,aa))
t.wsqs = array(0,dim=c(j,aa))

for( a in 1:aa){
  for ( j in 1:99){
    for( i in 1:q) {
      
      t.g[i,j,a]  = qgengamma.orig(j/100, sig.g[i,a], r0.g[i,a] , k[i,a])
      t.n[i,j,a]  = qlnorm(j/100, r0.n[i,a], sig.n[i,a])
      t.w[i,j,a]  = qweibull(j/100, sig.e[i,a], (r0.e[i,a]))
    }
    t.ghat[j,a] = sum(t.g[,j,a])/q
    t.wbar[j,a] = sum(t.w[,j,a])/q
    t.nbar[j,a] = sum(t.n[,j,a])/q
    RBN[j,a]= (t.nbar[j,a] - t.ghat[j,a])/t.ghat[j,a]
    RBW[j,a]= (t.wbar[j,a] - t.ghat[j,a])/t.ghat[j,a]  
  }
}

for ( a in 1:aa){
  for ( j in 1:99){
    for ( i in 1:q){
      msg[i,j,a] = (t.g[i,j,a] - t.ghat[j,a])^2 
      msw[i,j,a] = (t.w[i,j,a] - t.ghat[j,a])^2
      msn[i,j,a] = (t.n[i,j,a] - t.ghat[j,a])^2
    }
    RVg[j,a]=sum(msg[,j,a])
    RVw[j,a]=sum(msw[,j,a])
    RVn[j,a]=sum(msn[,j,a])
    RVwg[j,a] = RVw[j,a]/RVg[j,a]
    RVng[j,a] = RVn[j,a]/RVg[j,a]
  }
}

#plots 7 & 8
x = seq(0,95,5)
par(mfrow =c(1,2))

matplot(cbind(RBW[5:99,1:4],Rbiasgeneral[5:99,2],0) , type = "l", xlab = "(a)   p Quantile", ylab = "RB", main = "weibull" , cex.lab=1.1, cex.axis=1.1, xaxt = "n")
axis(side = 1, at = x,labels = seq(5,100,5))
legend("bottom",c("35","70","140","280","theoretical"),col = 1:5,lty = c(1, 1, 1), merge = TRUE,cex=0.9, title = "sample size")
matplot(cbind(RBN[5:95,1:4],Rbiasgeneral[5:95,3],0) , type = "l", xlab = "(b)   p Quantile", ylab = "RB", main = "Lognormal" , cex.lab=1.1, cex.axis=1.1, xaxt = "n")
axis(side = 1, at = x,labels = seq(5,100,5))
legend("top",c("35","70","140","280", "theoretical"),col = 1:5,lty = c(1, 1, 1), merge = TRUE,cex=0.9, title = "sample size")

y = matrix(c(5,10,20,40,125))
par(mfrow =c(2,2))
for ( i in 1:4){
  matplot(cbind(RBW[5:95,i], RBN[5:95,i],0) , type = "l", xlab = "p Quantile", ylab = "RV", main = paste("n=",y[i,]*7, sep= "") , cex.lab=1.1, cex.axis=1.1, xaxt = "n")
  axis(side = 1, at = x,labels = seq(5,100,5))
}

for ( i in 1:4){
  matplot(cbind(RVng[5:95,i], RVwg[5:95,i]) , type = "l", xlab = "p Quantile", ylab = "RV", main = paste("n=",y[i,]*7, sep= "") , cex.lab=1.1, cex.axis=1.1, xaxt = "n")
  axis(side = 1, at = x,labels = seq(5,100,5))
}
#############################################
# Effects of K

S = matrix(c(0.1,0.6,1),3,1)
K.g1 = matrix(0,15,1)
sigma.g1 = matrix(0,15,1)
r0g1 = matrix(0,15,1)
sigmae1 = matrix(0,15,1)
sigman1 = matrix(0,15,1)
a0.e1 = matrix(0,15,1)
a0.n1 = matrix(0,15,1)
a1.e1 = matrix(0,15,1)
a1.n1 = matrix(0,15,1)
v11 = matrix(0,99,15)
v21 = matrix(0,99,15)
v31 = matrix(0,99,15)
Rbiasgeneral1 = array(0, dim = c(99,15,2))
Rbiasgenerals1 = array(0, dim = c(99,15,2))

library(flexsurv)
for ( i in 1:15){
  G1 = matrix(c(0.61,1.55,i,-0.8),4,1)
  K.g1[i,] = (G1[3,1])
  sigma.g1[i,] = 1/G1[1,1]
  r0g1[i,] = log(G1[2,1]) 
  m.g1 = (r0g1[i,] + (S)*G1[4,1]) 
  
  neg.loglike.e  = function(teta) {
    log.beta= teta[3]; sigma = exp(log.beta)
    log.r0  = teta[1]; r0    = exp(log.r0)
    log.r1  = teta[2]; r1    = (log.r1)
    mu = r0 + r1*S
    g = exp(lgamma(sigma.g1[i,]/sigma + K.g1[i,]))/exp(lgamma(K.g1[i,]))
    (4/7)*log(sigma) + (4/7)*(mu[1,] - m.g1[1,] - sigma.g1[i,]*digamma(K.g1[i,]))/sigma + (4/7)*g*sum(exp(-(mu[1,] - m.g[1,])/sigma))+
      (2/7)*log(sigma) + (2/7)*(mu[2,] - m.g1[2,] - sigma.g1[i,]*digamma(K.g1[i,]))/sigma + (2/7)*g*sum(exp(-(mu[2,] - m.g[2,])/sigma))+
      (1/7)*log(sigma) + (1/7)*(mu[3,] - m.g1[3,] - sigma.g1[i,]*digamma(K.g1[i,]))/sigma + (1/7)*g*sum(exp(-(mu[3,] - m.g[3,])/sigma))
  }
  fitt.e   = optim(c(1,0,1),neg.loglike.e, hessian = TRUE)
  (sigmae1[i,] = exp(fitt.e$par[3]))
  (a0.e1[i,]  = exp(fitt.e$par[1]))
  (a1.e1[i,]  = (fitt.e$par[2]))
  
  neg.loglike.n  = function(teta) {
    log.beta= teta[3]; sigma = exp(log.beta)
    log.r0  = teta[1]; r0    = exp(log.r0)
    log.r1  = teta[2]; r1    = (log.r1)
    mu = r0 + r1*S
    (4/7)*log(sigma) + (4/7)*((mu[1,] - m.g1[1,] - sigma.g1[i,]*digamma(K.g1[i,]))^2 + sigma.g1[i,]^2*trigamma(K.g1[i,]))/(2*(sigma)^2)+
      (2/7)*log(sigma) + (2/7)*((mu[2,] - m.g1[2,] - sigma.g1[i,]*digamma(K.g1[i,]))^2 + sigma.g1[i,]^2*trigamma(K.g1[i,]))/(2*(sigma)^2)+
      (1/7)*log(sigma) + (1/7)*((mu[3,] - m.g1[3,] - sigma.g1[i,]*digamma(K.g1[i,]))^2 + sigma.g1[i,]^2*trigamma(K.g1[i,]))/(2*(sigma)^2)
  }
  fitt.n  = optim(c(1,0,1), neg.loglike.n, hessian = TRUE)
  (sigman1[i,] = exp(fitt.n$par[3]))
  (a0.n1[i,]  = exp(fitt.n$par[1]))
  (a1.n1[i,]  = (fitt.n$par[2]))
  
  c = seq(0.01,0.99, by =0.01)
  v11[,i] = qgengamma.orig(c, G1[1,1],G1[2,1] ,G1[3,1])
  v21[,i] = qlnorm(c, mean = a0.n1[i,] , sd = sigman1[i,])
  v31[,i] = qweibull(c, 1/sigmae1[i,], exp(a0.e1[i,]))
  
  Rbiasgeneral1[,i,1] = (v31[,i]-v11[,i])
  Rbiasgeneral1[,i,2] = (v21[,i]-v11[,i])
  Rbiasgenerals1[,i,1] = (v31[,i]-v11[,i])^2
  Rbiasgenerals1[,i,2] = (v21[,i]-v11[,i])^2
}

# plots 5 & 6
x = c(1,2,4,7,10,15)
y = c(2,4,7,10,15)
par(mfrow =c(2,3))
for ( i in 1:6){
  matplot(cbind(Rbiasgeneral1[,x[i],2], Rbiasgeneral1[,x[i],1], 0), type ="l", pch = 20, xlab = "(a)   100% p", ylab = "Bias", cex.lab=1.1, cex.axis=1.1,  main = paste("k=", x[i] , sep=""), ylim = c(-30,120))
}
for ( i in 1:6){
  matplot(cbind(Rbiasgenerals1[,x[i],2], Rbiasgenerals1[,x[i],1]), type ="l", pch = 20, xlab = "(a)   100% p", ylab = "MSE", cex.lab=1.1, cex.axis=1.1, main = paste("k=", x[i] , sep=""))
}